@@ -43,7 +43,7 @@ def _print_exception(e, indent=''):
             _print_exception(a, indent + '        ')
 
 
-class ResponseContextManager(object):
+class ResponseContextManager:
     """
     A context manager that closes a requests.Response when done.
     """
@@ -58,7 +58,7 @@ class ResponseContextManager(object):
         self.response.close()
 
 
-class HttpCallback(object):
+class HttpCallback:
     """
     A callback object that does nothing.  Overrides pre_request
     and/or post_request as desired.
@@ -128,7 +128,7 @@ class ClockSkewHook(HttpCallback):
             raise ClockSkew(skew_seconds)
 
 
-class B2Http(object):
+class B2Http:
     """
     A wrapper for the requests module.  Provides the operations
     needed to access B2, and handles retrying when the returned
@@ -154,7 +154,7 @@ class B2Http(object):
     """
 
     # timeout for HTTP GET/POST requests
-    TIMEOUT = 900  # 15 minutes as server-side copy can take time
+    TIMEOUT = 1200  # 20 minutes as server-side copy can take time
 
     def __init__(self, api_config: B2HttpApiConfig = DEFAULT_HTTP_API_CONFIG):
         """
@@ -346,6 +346,11 @@ class B2Http(object):
             if response.status_code not in [200, 206]:
                 # Decode the error object returned by the service
                 error = json.loads(response.content.decode('utf-8')) if response.content else {}
+                extra_error_keys = error.keys() - ('code', 'status', 'message')
+                if extra_error_keys:
+                    logger.debug(
+                        'received error has extra (unsupported) keys: %s', extra_error_keys
+                    )
                 raise interpret_b2_error(
                     int(error.get('status', response.status_code)),
                     error.get('code'),
