@@ -8,7 +8,6 @@
 #
 ######################################################################
 
-from abc import ABC, abstractmethod
 from typing import Dict, Optional, Union, Tuple, TYPE_CHECKING
 
 from .encryption.setting import EncryptionSetting, EncryptionSettingFactory
@@ -16,13 +15,16 @@ from .http_constants import FILE_INFO_HEADER_PREFIX_LOWER, SRC_LAST_MODIFIED_MIL
 from .file_lock import FileRetentionSetting, LegalHold, NO_RETENTION_FILE_SETTING
 from .progress import AbstractProgressListener
 from .utils.range_ import Range
+from .utils import b2_url_decode
 
 if TYPE_CHECKING:
     from .api import B2Api
     from .transfer.inbound.downloaded_file import DownloadedFile
 
+UNVERIFIED_CHECKSUM_PREFIX = 'unverified:'
 
-class BaseFileVersion(ABC):
+
+class BaseFileVersion:
     """
     Base class for representing file metadata in B2 cloud.
 
@@ -35,6 +37,7 @@ class BaseFileVersion(ABC):
         'size',
         'content_type',
         'content_sha1',
+        'content_sha1_verified',
         'file_info',
         'upload_timestamp',
         'server_side_encryption',
@@ -62,24 +65,52 @@ class BaseFileVersion(ABC):
         self.file_name = file_name
         self.size = size
         self.content_type = content_type
-        self.content_sha1 = content_sha1
+        self.content_sha1, self.content_sha1_verified = self._decode_content_sha1(content_sha1)
         self.file_info = file_info or {}
         self.upload_timestamp = upload_timestamp
         self.server_side_encryption = server_side_encryption
-        self.legal_hold = legal_hold
         self.file_retention = file_retention
+        self.legal_hold = legal_hold
 
         if SRC_LAST_MODIFIED_MILLIS in self.file_info:
             self.mod_time_millis = int(self.file_info[SRC_LAST_MODIFIED_MILLIS])
         else:
             self.mod_time_millis = self.upload_timestamp
 
-    @abstractmethod
-    def _clone(self, **new_attributes):
+    @classmethod
+    def _decode_content_sha1(cls, content_sha1):
+        if content_sha1.startswith(UNVERIFIED_CHECKSUM_PREFIX):
+            return content_sha1[len(UNVERIFIED_CHECKSUM_PREFIX):], False
+        return content_sha1, True
+
+    @classmethod
+    def _encode_content_sha1(cls, content_sha1, content_sha1_verified):
+        if not content_sha1_verified:
+            return '%s%s' % (UNVERIFIED_CHECKSUM_PREFIX, content_sha1)
+        return content_sha1
+
+    def _clone(self, **new_attributes: Dict[str, object]):
         """
         Create new instance based on the old one, overriding attributes with :code:`new_attributes`
         (only applies to arguments passed to __init__)
         """
+        args = self._get_args_for_clone()
+        return self.__class__(**{**args, **new_attributes})
+
+    def _get_args_for_clone(self):
+        return {
+            'api': self.api,
+            'id_': self.id_,
+            'file_name': self.file_name,
+            'size': self.size,
+            'content_type': self.content_type,
+            'content_sha1': self._encode_content_sha1(self.content_sha1, self.content_sha1_verified),
+            'file_info': self.file_info,
+            'upload_timestamp': self.upload_timestamp,
+            'server_side_encryption': self.server_side_encryption,
+            'file_retention': self.file_retention,
+            'legal_hold': self.legal_hold,
+        }  # yapf: disable
 
     def as_dict(self):
         """ represents the object as a dict which looks almost exactly like the raw api output for upload/list """
@@ -99,7 +130,9 @@ class BaseFileVersion(ABC):
         if self.content_type is not None:
             result['contentType'] = self.content_type
         if self.content_sha1 is not None:
-            result['contentSha1'] = self.content_sha1
+            result['contentSha1'] = self._encode_content_sha1(
+                self.content_sha1, self.content_sha1_verified
+            )
 
         return result
 
@@ -197,25 +230,17 @@ class FileVersion(BaseFileVersion):
             legal_hold=legal_hold,
         )
 
-    def _clone(self, **new_attributes: Dict[str, object]):
-        args = {
-            'api': self.api,
-            'id_': self.id_,
-            'file_name': self.file_name,
-            'size': self.size,
-            'content_type': self.content_type,
-            'content_sha1': self.content_sha1,
-            'file_info': self.file_info,
-            'upload_timestamp': self.upload_timestamp,
-            'account_id': self.account_id,
-            'bucket_id': self.bucket_id,
-            'action': self.action,
-            'content_md5': self.content_md5,
-            'server_side_encryption': self.server_side_encryption,
-            'file_retention': self.file_retention,
-            'legal_hold': self.legal_hold,
-        }
-        return self.__class__(**{**args, **new_attributes})
+    def _get_args_for_clone(self):
+        args = super()._get_args_for_clone()
+        args.update(
+            {
+                'account_id': self.account_id,
+                'bucket_id': self.bucket_id,
+                'action': self.action,
+                'content_md5': self.content_md5,
+            }
+        )
+        return args
 
     def as_dict(self):
         result = super().as_dict()
@@ -307,35 +332,29 @@ class DownloadVersion(BaseFileVersion):
             legal_hold=legal_hold,
         )
 
-    def _clone(self, **new_attributes: Dict[str, object]):
-        args = {
-            'api': self.api,
-            'id_': self.id_,
-            'file_name': self.file_name,
-            'size': self.size,
-            'content_type': self.content_type,
-            'content_sha1': self.content_sha1,
-            'file_info': self.file_info,
-            'upload_timestamp': self.upload_timestamp,
-            'server_side_encryption': self.server_side_encryption,
-            'range_': self.range_,
-            'content_disposition': self.content_disposition,
-            'content_length': self.content_length,
-            'content_language': self.content_language,
-            'expires': self._expires,
-            'cache_control': self._cache_control,
-            'content_encoding': self.content_encoding,
-            'file_retention': self.file_retention,
-            'legal_hold': self.legal_hold,
-        }
-        return self.__class__(**{**args, **new_attributes})
+    def _get_args_for_clone(self):
+        args = super()._get_args_for_clone()
+        args.update(
+            {
+                'range_': self.range_,
+                'content_disposition': self.content_disposition,
+                'content_length': self.content_length,
+                'content_language': self.content_language,
+                'expires': self._expires,
+                'cache_control': self._cache_control,
+                'content_encoding': self.content_encoding,
+            }
+        )
+        return args
 
 
-class FileVersionFactory(object):
+class FileVersionFactory:
     """
     Construct :py:class:`b2sdk.v2.FileVersion` objects from api responses.
     """
 
+    FILE_VERSION_CLASS = FileVersion
+
     def __init__(self, api: 'B2Api'):
         self.api = api
 
@@ -392,8 +411,7 @@ class FileVersionFactory(object):
         file_retention = FileRetentionSetting.from_file_version_dict(file_version_dict)
 
         legal_hold = LegalHold.from_file_version_dict(file_version_dict)
-
-        return FileVersion(
+        return self.FILE_VERSION_CLASS(
             self.api,
             id_,
             file_name,
@@ -412,7 +430,7 @@ class FileVersionFactory(object):
         )
 
 
-class DownloadVersionFactory(object):
+class DownloadVersionFactory:
     """
     Construct :py:class:`b2sdk.v2.DownloadVersion` objects from download headers.
     """
@@ -435,7 +453,7 @@ class DownloadVersionFactory(object):
         for header_name, header_value in headers.items():
             if header_name[:prefix_len].lower() == FILE_INFO_HEADER_PREFIX_LOWER:
                 file_info_key = header_name[prefix_len:]
-                file_info[file_info_key] = header_value
+                file_info[file_info_key] = b2_url_decode(header_value)
         return file_info
 
     def from_response_headers(self, headers):
@@ -450,7 +468,7 @@ class DownloadVersionFactory(object):
         return DownloadVersion(
             api=self.api,
             id_=headers['x-bz-file-id'],
-            file_name=headers['x-bz-file-name'],
+            file_name=b2_url_decode(headers['x-bz-file-name']),
             size=size,
             content_type=headers['content-type'],
             content_sha1=headers['x-bz-content-sha1'],
@@ -469,7 +487,7 @@ class DownloadVersionFactory(object):
         )
 
 
-class FileIdAndName(object):
+class FileIdAndName:
     """
     A structure which represents a B2 cloud file with just `file_name` and `fileId` attributes.
 
