@@ -18,13 +18,8 @@ from ..file_version import FileVersion
 
 logger = logging.getLogger(__name__)
 
-try:  # python 3.5 and 3.6 compatibility
-    regex_class = re.Pattern
-except AttributeError:
-    regex_class = re._pattern_type
 
-
-class RegexSet(object):
+class RegexSet:
     """
     Hold a (possibly empty) set of regular expressions and know how to check
     whether a string matches any of them.
@@ -77,7 +72,7 @@ def convert_dir_regex_to_dir_prefix_regex(dir_regex):
         return dir_regex + r'.*?/'
 
 
-class IntegerRange(object):
+class IntegerRange:
     """
     Hold a range of two integers. If the range value is None, it indicates that
     the value should be treated as -Inf (for begin) or +Inf (for end).
@@ -110,7 +105,7 @@ class IntegerRange(object):
         return ge_begin and le_end
 
 
-class ScanPoliciesManager(object):
+class ScanPoliciesManager:
     """
     Policy object used when scanning folders for syncing, used to decide
     which files to include in the list of files to be synced.
@@ -126,9 +121,9 @@ class ScanPoliciesManager(object):
 
     def __init__(
         self,
-        exclude_dir_regexes: Iterable[Union[str, regex_class]] = tuple(),
-        exclude_file_regexes: Iterable[Union[str, regex_class]] = tuple(),
-        include_file_regexes: Iterable[Union[str, regex_class]] = tuple(),
+        exclude_dir_regexes: Iterable[Union[str, re.Pattern]] = tuple(),
+        exclude_file_regexes: Iterable[Union[str, re.Pattern]] = tuple(),
+        include_file_regexes: Iterable[Union[str, re.Pattern]] = tuple(),
         exclude_all_symlinks: bool = False,
         exclude_modified_before: Optional[int] = None,
         exclude_modified_after: Optional[int] = None,
