@@ -48,6 +48,7 @@ from apiver_deps import MetadataDirectiveMode
 from apiver_deps import Part
 from apiver_deps import AbstractProgressListener
 from apiver_deps import StubAccountInfo, RawSimulator, BucketSimulator, FakeResponse, FileSimulator
+from apiver_deps import AbstractDownloader
 from apiver_deps import ParallelDownloader
 from apiver_deps import Range
 from apiver_deps import SimpleDownloader
@@ -175,12 +176,15 @@ def bucket_ls(bucket, *args, show_versions=False, **kwargs):
 class TestCaseWithBucket(TestBase):
     RAW_SIMULATOR_CLASS = RawSimulator
 
+    def get_api(self):
+        return B2Api(
+            self.account_info, api_config=B2HttpApiConfig(_raw_api_class=self.RAW_SIMULATOR_CLASS)
+        )
+
     def setUp(self):
         self.bucket_name = 'my-bucket'
         self.account_info = StubAccountInfo()
-        self.api = B2Api(
-            self.account_info, api_config=B2HttpApiConfig(_raw_api_class=self.RAW_SIMULATOR_CLASS)
-        )
+        self.api = self.get_api()
         self.simulator = self.api.session.raw_api
         (self.account_id, self.master_key) = self.simulator.create_account()
         self.api.authorize_account('production', self.account_id, self.master_key)
@@ -218,10 +222,26 @@ class TestCaseWithBucket(TestBase):
             fragments.append(fragment)
         return b''.join(fragments)
 
+    def _check_file_contents(self, file_name, expected_contents):
+        contents = self._download_file(file_name)
+        self.assertEqual(expected_contents, contents)
+
+    def _download_file(self, file_name):
+        with FileSimulator.dont_check_encryption():
+            if apiver_deps.V <= 1:
+                download = DownloadDestBytes()
+                self.bucket.download_file_by_name(file_name, download)
+                return download.get_bytes_written()
+            else:
+                with io.BytesIO() as bytes_io:
+                    downloaded_file = self.bucket.download_file_by_name(file_name)
+                    downloaded_file.save(bytes_io)
+                    return bytes_io.getvalue()
+
 
 class TestReauthorization(TestCaseWithBucket):
     def testCreateBucket(self):
-        class InvalidAuthTokenWrapper(object):
+        class InvalidAuthTokenWrapper:
             def __init__(self, original_function):
                 self.__original_function = original_function
                 self.__name__ = original_function.__name__
@@ -611,18 +631,53 @@ class TestListVersions(TestCaseWithBucket):
 
 
 class TestCopyFile(TestCaseWithBucket):
-    @pytest.mark.apiver(to_ver=1)
+    @classmethod
+    def _copy_function(cls, bucket):
+        if apiver_deps.V <= 1:
+            return bucket.copy_file
+        else:
+            return bucket.copy
+
+    @pytest.mark.apiver(from_ver=2)
+    def test_copy_big(self):
+        data = b'HelloWorld' * 100
+        for i in range(10):
+            data += bytes(':#' + str(i) + '$' + 'abcdefghijklmnopqrstuvwx' * 4, 'ascii')
+        file_info = self.bucket.upload_bytes(data, 'file1')
+
+        self.bucket.copy(
+            file_info.id_,
+            'file2',
+            min_part_size=200,
+            max_part_size=400,
+        )
+        self._check_file_contents('file2', data)
+
     def test_copy_without_optional_params(self):
         file_id = self._make_file()
-        self.bucket.copy_file(file_id, 'hello_new.txt')
-        expected = [('hello.txt', 11, 'upload', None), ('hello_new.txt', 11, 'copy', None)]
+        if apiver_deps.V <= 1:
+            f = self.bucket.copy_file(file_id, 'hello_new.txt')
+            assert f['action'] == 'copy'
+        else:
+            f = self.bucket.copy(file_id, 'hello_new.txt')
+            assert f.action == 'copy'
+        expected = [('hello.txt', 11, 'upload', None), ('hello_new.txt', 11, 'upload', None)]
         self.assertBucketContents(expected, '', show_versions=True)
 
-    @pytest.mark.apiver(to_ver=1)
     def test_copy_with_range(self):
         file_id = self._make_file()
-        self.bucket.copy_file(file_id, 'hello_new.txt', bytes_range=(3, 9))
-        expected = [('hello.txt', 11, 'upload', None), ('hello_new.txt', 6, 'copy', None)]
+        #data = b'hello world'
+        #            3456789
+        if apiver_deps.V <= 1:
+            self.bucket.copy_file(
+                file_id,
+                'hello_new.txt',
+                bytes_range=(3, 9),
+            )  # inclusive, confusingly
+        else:
+            self.bucket.copy(file_id, 'hello_new.txt', offset=3, length=7)
+        self._check_file_contents('hello_new.txt', b'lo worl')
+        expected = [('hello.txt', 11, 'upload', None), ('hello_new.txt', 7, 'upload', None)]
         self.assertBucketContents(expected, '', show_versions=True)
 
     @pytest.mark.apiver(to_ver=1)
@@ -673,7 +728,7 @@ class TestCopyFile(TestCaseWithBucket):
         )
         expected = [
             ('hello.txt', 11, 'upload', 'b2/x-auto', None),
-            ('hello_new.txt', 11, 'copy', 'text/plain', None),
+            ('hello_new.txt', 11, 'upload', 'text/plain', None),
         ]
         actual = [
             (info.file_name, info.size, info.action, info.content_type, folder)
@@ -681,15 +736,22 @@ class TestCopyFile(TestCaseWithBucket):
         ]
         self.assertEqual(expected, actual)
 
-    @pytest.mark.apiver(to_ver=1)
     def test_copy_with_unsatisfied_range(self):
         file_id = self._make_file()
         try:
-            self.bucket.copy_file(
-                file_id,
-                'hello_new.txt',
-                bytes_range=(12, 15),
-            )
+            if apiver_deps.V <= 1:
+                self.bucket.copy_file(
+                    file_id,
+                    'hello_new.txt',
+                    bytes_range=(12, 15),
+                )
+            else:
+                self.bucket.copy(
+                    file_id,
+                    'hello_new.txt',
+                    offset=12,
+                    length=3,
+                )
             self.fail('should have raised UnsatisfiableRange')
         except UnsatisfiableRange as e:
             self.assertEqual(
@@ -699,11 +761,10 @@ class TestCopyFile(TestCaseWithBucket):
         expected = [('hello.txt', 11, 'upload', None)]
         self.assertBucketContents(expected, '', show_versions=True)
 
-    @pytest.mark.apiver(to_ver=1)
     def test_copy_with_different_bucket(self):
         source_bucket = self.api.create_bucket('source-bucket', 'allPublic')
         file_id = self._make_file(source_bucket)
-        self.bucket.copy_file(file_id, 'hello_new.txt')
+        self._copy_function(self.bucket)(file_id, 'hello_new.txt')
 
         def ls(bucket):
             return [
@@ -713,7 +774,7 @@ class TestCopyFile(TestCaseWithBucket):
 
         expected = [('hello.txt', 11, 'upload', None)]
         self.assertEqual(expected, ls(source_bucket))
-        expected = [('hello_new.txt', 11, 'copy', None)]
+        expected = [('hello_new.txt', 11, 'upload', None)]
         self.assertBucketContents(expected, '', show_versions=True)
 
     def test_copy_retention(self):
@@ -725,7 +786,8 @@ class TestCopyFile(TestCaseWithBucket):
                         file_id,
                         'copied_file',
                         file_retention=FileRetentionSetting(RetentionMode.COMPLIANCE, 100),
-                        legal_hold=LegalHold.ON
+                        legal_hold=LegalHold.ON,
+                        max_part_size=400,
                     )
                     self.assertEqual(
                         FileRetentionSetting(RetentionMode.COMPLIANCE, 100),
@@ -1105,6 +1167,42 @@ class TestUpload(TestCaseWithBucket):
             self.bucket.upload_local_file(path, 'file1')
             self._check_file_contents('file1', data)
 
+    def test_upload_local_large_file_over_10k_parts(self):
+        pytest.skip('this test is really slow and impedes development')  # TODO: fix it
+        with TempDir() as d:
+            path = os.path.join(d, 'file1')
+            data = self._make_data(self.simulator.MIN_PART_SIZE * 10001)  # 2MB on the simulator
+            write_file(path, data)
+            self.bucket.upload_local_file(path, 'file1')
+            self._check_file_contents('file1', data)
+
+    def test_create_file_over_10k_parts(self):
+        data = b'hello world' * 20000
+
+        f1_id = self.bucket.upload_bytes(data, 'f1').id_
+        with TempDir() as d:
+            write_intents = [
+                WriteIntent(
+                    CopySource(f1_id, length=len(data), offset=0),
+                    destination_offset=0,
+                )
+            ] * 10
+            created_file = self.bucket.create_file(
+                write_intents,
+                file_name='created_file',
+                min_part_size=10,
+                max_part_size=200,
+            )
+            self.assertIsInstance(created_file, VFileVersionInfo)
+            actual = (
+                created_file.id_,
+                created_file.file_name,
+                created_file.size,
+                created_file.server_side_encryption,
+            )
+            expected = ('9998', 'created_file', len(data), SSE_NONE)
+            self.assertEqual(expected, actual)
+
     def test_upload_large_resume(self):
         part_size = self.simulator.MIN_PART_SIZE
         data = self._make_data(part_size * 3)
@@ -1217,22 +1315,6 @@ class TestUpload(TestCaseWithBucket):
             len(part_data), hex_sha1_of_bytes(part_data), part_stream
         )
 
-    def _check_file_contents(self, file_name, expected_contents):
-        contents = self._download_file(file_name)
-        self.assertEqual(expected_contents, contents)
-
-    def _download_file(self, file_name):
-        with FileSimulator.dont_check_encryption():
-            if apiver_deps.V <= 1:
-                download = DownloadDestBytes()
-                self.bucket.download_file_by_name(file_name, download)
-                return download.get_bytes_written()
-            else:
-                with io.BytesIO() as bytes_io:
-                    downloaded_file = self.bucket.download_file_by_name(file_name)
-                    downloaded_file.save(bytes_io)
-                    return bytes_io.getvalue()
-
 
 class TestConcatenate(TestCaseWithBucket):
     def _create_remote(self, sources, file_name, encryption=None):
@@ -1316,7 +1398,7 @@ class TestCreateFileStream(TestConcatenate):
         )
 
 
-class DownloadTestsBase(object):
+class DownloadTestsBase:
     DATA = NotImplemented
 
     def setUp(self):
@@ -1637,7 +1719,7 @@ class DownloadTests(DownloadTestsBase):
 # download empty file
 
 
-class EmptyFileDownloadScenarioMixin(object):
+class EmptyFileDownloadScenarioMixin:
     """ use with DownloadTests, but not for TestDownloadParallel as it does not like empty files """
 
     def test_download_by_name_empty_file(self):
@@ -1646,6 +1728,25 @@ class EmptyFileDownloadScenarioMixin(object):
         self._verify('')
 
 
+class UnverifiedChecksumDownloadScenarioMixin:
+    """ use with DownloadTests """
+
+    def test_download_by_name_unverified_checksum(self):
+        with TempDir() as d:
+            path = os.path.join(d, 'file1')
+            data = b'hello world'
+            write_file(path, data)
+            file_info = self.bucket.upload_local_file(path, 'file1')
+            simulated_file = list(self.simulator.bucket_name_to_bucket.values()
+                                 )[0].file_id_to_file[file_info.id_]
+            simulated_file.content_sha1 = 'unverified:' + simulated_file.content_sha1
+            #, sha1_sum='unverified:2aae6c35c94fcfb415dbe95f408b9ce91ee846ed')
+
+            self.download_file_by_name('file1', progress_listener=self.progress_listener)
+
+            self._verify('hello world')
+
+
 # actual tests
 
 # test choosing strategy
@@ -1655,13 +1756,15 @@ class EmptyFileDownloadScenarioMixin(object):
 class TestChooseStrategy(TestCaseWithBucket):
     def test_choose_strategy(self):
         file_version = self.bucket.upload_bytes(b'hello world' * 8, 'file1')
+        download_manager = self.bucket.api.services.download_manager
         parallel_downloader = ParallelDownloader(
             force_chunk_size=1,
             max_streams=32,
             min_part_size=16,
+            thread_pool=download_manager._thread_pool,
         )
-        simple_downloader = self.bucket.api.services.download_manager.strategies[1]
-        self.bucket.api.services.download_manager.strategies = [
+        simple_downloader = download_manager.strategies[1]
+        download_manager.strategies = [
             parallel_downloader,
             simple_downloader,
         ]
@@ -1690,18 +1793,27 @@ class TestDownloadDefault(DownloadTests, EmptyFileDownloadScenarioMixin, TestCas
     pass
 
 
-class TestDownloadSimple(DownloadTests, EmptyFileDownloadScenarioMixin, TestCaseWithBucket):
+class TestDownloadSimple(
+    DownloadTests,
+    UnverifiedChecksumDownloadScenarioMixin,
+    EmptyFileDownloadScenarioMixin,
+    TestCaseWithBucket,
+):
     def setUp(self):
         super(TestDownloadSimple, self).setUp()
-        self.bucket.api.services.download_manager.strategies = [
-            SimpleDownloader(force_chunk_size=20,)
-        ]
+        download_manager = self.bucket.api.services.download_manager
+        download_manager.strategies = [SimpleDownloader(force_chunk_size=20)]
 
 
-class TestDownloadParallel(DownloadTests, TestCaseWithBucket):
+class TestDownloadParallel(
+    DownloadTests,
+    UnverifiedChecksumDownloadScenarioMixin,
+    TestCaseWithBucket,
+):
     def setUp(self):
         super(TestDownloadParallel, self).setUp()
-        self.bucket.api.services.download_manager.strategies = [
+        download_manager = self.bucket.api.services.download_manager
+        download_manager.strategies = [
             ParallelDownloader(
                 force_chunk_size=2,
                 max_streams=999,
@@ -1715,7 +1827,8 @@ class TestDownloadParallelALotOfStreams(DownloadTestsBase, TestCaseWithBucket):
 
     def setUp(self):
         super(TestDownloadParallelALotOfStreams, self).setUp()
-        self.bucket.api.services.download_manager.strategies = [
+        download_manager = self.bucket.api.services.download_manager
+        download_manager.strategies = [
             # this should produce 32 streams with 16 single byte writes
             # so we increase probability of non-sequential writes as much as possible
             # with great help from random sleeps in FakeResponse
@@ -1723,6 +1836,7 @@ class TestDownloadParallelALotOfStreams(DownloadTestsBase, TestCaseWithBucket):
                 force_chunk_size=1,
                 max_streams=32,
                 min_part_size=16,
+                thread_pool=download_manager._thread_pool,
             ),
         ]
 
@@ -1763,18 +1877,201 @@ class TestCaseWithTruncatedDownloadBucket(TestCaseWithBucket):
 class TestTruncatedDownloadSimple(DownloadTests, TestCaseWithTruncatedDownloadBucket):
     def setUp(self):
         super(TestTruncatedDownloadSimple, self).setUp()
-        self.bucket.api.services.download_manager.strategies = [
-            SimpleDownloader(force_chunk_size=20,)
-        ]
+        download_manager = self.bucket.api.services.download_manager
+        download_manager.strategies = [SimpleDownloader(force_chunk_size=20)]
 
 
 class TestTruncatedDownloadParallel(DownloadTests, TestCaseWithTruncatedDownloadBucket):
     def setUp(self):
         super(TestTruncatedDownloadParallel, self).setUp()
-        self.bucket.api.services.download_manager.strategies = [
+        download_manager = self.bucket.api.services.download_manager
+        download_manager.strategies = [
             ParallelDownloader(
                 force_chunk_size=3,
                 max_streams=2,
                 min_part_size=2,
             )
         ]
+
+
+class DummyDownloader(AbstractDownloader):
+    def download(self, *args, **kwargs):
+        pass
+
+
+@pytest.mark.parametrize(
+    'min_chunk_size,max_chunk_size,content_length,align_factor,expected_chunk_size',
+    [
+        (10, 100, 1000 * 9, 8, 8),  # min_chunk_size aligned
+        (10, 100, 1000 * 17, 8, 16),  # content_length // 1000 aligned
+        (10, 100, 1000 * 108, 8, 96),  # max_chunk_size // 1000 aligned
+        (10, 100, 1000 * 9, 100, 100),  # max_chunk_size/align_factor
+        (10, 100, 1000 * 17, 100, 100),  # max_chunk_size/align_factor
+        (10, 100, 1000 * 108, 100, 100),  # max_chunk_size/align_factor
+        (10, 100, 1, 100, 100),  # max_chunk_size/align_factor
+    ],
+)
+def test_downloader_get_chunk_size(
+    min_chunk_size, max_chunk_size, content_length, align_factor, expected_chunk_size
+):
+    downloader = DummyDownloader(
+        min_chunk_size=min_chunk_size,
+        max_chunk_size=max_chunk_size,
+        align_factor=align_factor,
+    )
+    assert downloader._get_chunk_size(content_length) == expected_chunk_size
+
+
+@pytest.mark.apiver(from_ver=2)
+class TestDownloadTuneWriteBuffer(DownloadTestsBase, TestCaseWithBucket):
+    ALIGN_FACTOR = 123
+    DATA = 'abc' * 4096
+
+    def get_api(self):
+        return B2Api(
+            self.account_info,
+            api_config=B2HttpApiConfig(_raw_api_class=self.RAW_SIMULATOR_CLASS),
+            save_to_buffer_size=self.ALIGN_FACTOR,
+        )
+
+    def test_get_chunk_size_alignment(self):
+        download_manager = self.bucket.api.services.download_manager
+        for downloader in download_manager.strategies:
+            assert downloader._get_chunk_size(len(self.DATA)) % self.ALIGN_FACTOR == 0
+
+    def test_buffering_in_save_to(self):
+        with TempDir() as d:
+            path = os.path.join(d, 'file2')
+            with mock.patch('b2sdk.transfer.inbound.downloaded_file.open') as mock_open:
+                mock_open.side_effect = open
+                self.bucket.download_file_by_id(self.file_version.id_).save_to(path)
+                mock_open.assert_called_once_with(path, mock.ANY, buffering=self.ALIGN_FACTOR)
+                with open(path, 'r') as f:
+                    contents = f.read()
+                    assert contents == self.DATA
+
+    def test_set_write_buffer_parallel_called_get_chunk_size(self):
+        self._check_called_on_downloader(
+            ParallelDownloader(
+                force_chunk_size=len(self.DATA) // 3,
+                max_streams=999,
+                min_part_size=len(self.DATA) // 3,
+            )
+        )
+
+    def test_set_write_buffer_simple_called_get_chunk_size(self):
+        self._check_called_on_downloader(SimpleDownloader(force_chunk_size=len(self.DATA) // 3))
+
+    def _check_called_on_downloader(self, downloader):
+        download_manager = self.bucket.api.services.download_manager
+        download_manager.strategies = [downloader]
+        orig_get_chunk_size = downloader._get_chunk_size
+        with mock.patch.object(downloader, '_get_chunk_size') as mock_get_chunk_size:
+            mock_get_chunk_size.side_effect = orig_get_chunk_size
+            self.download_file_by_id(self.file_version.id_)
+            self._verify(self.DATA, check_progress_listener=False)
+            assert mock_get_chunk_size.called
+
+
+@pytest.mark.apiver(from_ver=2)
+class TestDownloadNoHashChecking(DownloadTestsBase, TestCaseWithBucket):
+    DATA = 'abcdefghijklmnopqrs'
+
+    def get_api(self):
+        return B2Api(
+            self.account_info,
+            api_config=B2HttpApiConfig(_raw_api_class=self.RAW_SIMULATOR_CLASS),
+            check_download_hash=False,
+        )
+
+    def test_download_by_id_no_hash_checking(self):
+        downloaded_file = self.bucket.download_file_by_id(self.file_version.id_)
+        orig_validate_download = downloaded_file._validate_download
+        with mock.patch.object(downloaded_file, '_validate_download') as mocked_validate_download:
+            mocked_validate_download.side_effect = orig_validate_download
+            downloaded_file.save(self.bytes_io)
+            self._verify(self.DATA, check_progress_listener=False)
+            mocked_validate_download.assert_called_once_with(mock.ANY, '')
+            assert downloaded_file.download_version.content_sha1 != 'none'
+            assert downloaded_file.download_version.content_sha1 != ''
+
+
+class DecodeTestsBase(object):
+    def setUp(self):
+        super(DecodeTestsBase, self).setUp()
+        self.bucket.upload_bytes(
+            'Test File 1'.encode(), 'test.txt?foo=bar', file_infos={'custom_info': 'aaa?bbb'}
+        )
+        self.bucket.upload_bytes(
+            'Test File 2'.encode(), 'test.txt%3Ffoo=bar', file_infos={'custom_info': 'aaa%3Fbbb'}
+        )
+        self.bucket.upload_bytes('Test File 3'.encode(), 'test.txt%3Ffoo%3Dbar')
+        self.bucket.upload_bytes('Test File 4'.encode(), 'test.txt%253Ffoo%253Dbar')
+        self.bytes_io = io.BytesIO()
+        if apiver_deps.V <= 1:
+            self.download_dest = DownloadDestBytes()
+        else:
+            self.download_dest = None
+        self.progress_listener = StubProgressListener()
+
+    def _verify(self, expected_result, check_progress_listener=True):
+        self._assert_downloaded_data(expected_result)
+        if check_progress_listener:
+            valid, reason = self.progress_listener.is_valid_reason(
+                check_closed=False,
+                check_progress=False,
+                check_monotonic_progress=True,
+            )
+            assert valid, reason
+
+    def _assert_downloaded_data(self, expected_result):
+        if apiver_deps.V <= 1:
+            assert self.download_dest.get_bytes_written() == expected_result.encode()
+        else:
+            assert self.bytes_io.getvalue() == expected_result.encode()
+
+    def download_file_by_name(self, file_name, download_dest=None, **kwargs):
+        if apiver_deps.V <= 1:
+            self.bucket.download_file_by_name(
+                file_name, download_dest or self.download_dest, **kwargs
+            )
+        else:
+            self.bucket.download_file_by_name(file_name, **kwargs).save(self.bytes_io)
+
+
+class DecodeTests(DecodeTestsBase, TestCaseWithBucket):
+    def test_file_content_1(self):
+        self.download_file_by_name('test.txt?foo=bar', progress_listener=self.progress_listener)
+        self._verify("Test File 1")
+
+    def test_file_content_2(self):
+        self.download_file_by_name('test.txt%3Ffoo=bar', progress_listener=self.progress_listener)
+        self._verify("Test File 2")
+
+    def test_file_content_3(self):
+        self.download_file_by_name('test.txt%3Ffoo%3Dbar', progress_listener=self.progress_listener)
+        self._verify("Test File 3")
+
+    def test_file_content_4(self):
+        self.download_file_by_name(
+            'test.txt%253Ffoo%253Dbar', progress_listener=self.progress_listener
+        )
+        self._verify("Test File 4")
+
+    def test_file_info_1(self):
+        download_version = self.bucket.get_file_info_by_name('test.txt?foo=bar')
+        assert download_version.file_name == 'test.txt?foo=bar'
+        assert download_version.file_info['custom_info'] == 'aaa?bbb'
+
+    def test_file_info_2(self):
+        download_version = self.bucket.get_file_info_by_name('test.txt%3Ffoo=bar')
+        assert download_version.file_name == 'test.txt%3Ffoo=bar'
+        assert download_version.file_info['custom_info'] == 'aaa%3Fbbb'
+
+    def test_file_info_3(self):
+        download_version = self.bucket.get_file_info_by_name('test.txt%3Ffoo%3Dbar')
+        assert download_version.file_name == 'test.txt%3Ffoo%3Dbar'
+
+    def test_file_info_4(self):
+        download_version = self.bucket.get_file_info_by_name('test.txt%253Ffoo%253Dbar')
+        assert download_version.file_name == 'test.txt%253Ffoo%253Dbar'
